@@ -18,7 +18,7 @@ This script splits a convolutional model onto multiple GPUs, whilst using the in
 to balance across your GPUs.
 
 To run:
-python conv_model_sequential_example.py --accelerator ddp --gpus 4 --max_epochs 1  --batch_size 256 --use_ddp_sequential
+python conv_model_sequential_example.py --accelerator ddp --gpus 4 --max_epochs 1  --batch_size 256 --use_rpc_sequential
 """
 import math
 from argparse import ArgumentParser
@@ -32,23 +32,24 @@ import pytorch_lightning as pl
 from pl_examples import cli_lightning_logo
 from pytorch_lightning import Trainer
 from pytorch_lightning.metrics.functional import accuracy
-from pytorch_lightning.plugins.ddp_sequential_plugin import DDPSequentialPlugin
-from pytorch_lightning.utilities import BOLTS_AVAILABLE, FAIRSCALE_PIPE_AVAILABLE
+from pytorch_lightning.plugins import RPCSequentialPlugin
+from pytorch_lightning.utilities import _BOLTS_AVAILABLE, _FAIRSCALE_PIPE_AVAILABLE
 
-if BOLTS_AVAILABLE:
+if _BOLTS_AVAILABLE:
     import pl_bolts
     from pl_bolts.transforms.dataset_normalizations import cifar10_normalization
 
-
 #####################
 #      Modules      #
 #####################
 
 
 class Flatten(nn.Module):
+
     def forward(self, x):
         return x.view(x.size(0), -1)
 
+
 ###############################
 #       LightningModule       #
 ###############################
@@ -61,6 +62,7 @@ class LitResnet(pl.LightningModule):
       (sequential_module): Sequential(...)
     )
     """
+
     def __init__(self, lr=0.05, batch_size=32, manual_optimization=False):
         super().__init__()
 
@@ -90,9 +92,7 @@ class LitResnet(pl.LightningModule):
             nn.Conv2d(in_channels=256, out_channels=256, kernel_size=3, padding=1),
             nn.ReLU(inplace=False),
             nn.MaxPool2d(kernel_size=2, stride=2),
-
             Flatten(),
-
             nn.Dropout(p=0.1),
             nn.Linear(4096, 1024),
             nn.ReLU(inplace=False),
@@ -159,7 +159,8 @@ class LitResnet(pl.LightningModule):
                     optimizer,
                     0.1,
                     epochs=self.trainer.max_epochs,
-                    steps_per_epoch=math.ceil(45000 / self.hparams.batch_size)),
+                    steps_per_epoch=math.ceil(45000 / self.hparams.batch_size)
+                ),
                 'interval': 'step',
             }
         }
@@ -173,6 +174,7 @@ class LitResnet(pl.LightningModule):
 #     Instantiate Data Module   #
 #################################
 
+
 def instantiate_datamodule(args):
     train_transforms = torchvision.transforms.Compose([
         torchvision.transforms.RandomCrop(32, padding=4),
@@ -198,20 +200,21 @@ def instantiate_datamodule(args):
 
 if __name__ == "__main__":
     cli_lightning_logo()
+
+    assert _BOLTS_AVAILABLE, "Bolts is required for this example, install it via pip install pytorch-lightning-bolts"
+    assert _FAIRSCALE_PIPE_AVAILABLE, "FairScale and PyTorch 1.6 is required for this example."
+
     parser = ArgumentParser(description="Pipe Example")
-    parser.add_argument("--use_ddp_sequential", action="store_true")
+    parser.add_argument("--use_rpc_sequential", action="store_true")
     parser = Trainer.add_argparse_args(parser)
     parser = pl_bolts.datamodules.CIFAR10DataModule.add_argparse_args(parser)
     args = parser.parse_args()
 
-    assert BOLTS_AVAILABLE, "Bolts is required for this example, install it via pip install pytorch-lightning-bolts"
-    assert FAIRSCALE_PIPE_AVAILABLE, "FairScale and PyTorch 1.6 is required for this example."
-
     cifar10_dm = instantiate_datamodule(args)
 
     plugins = None
-    if args.use_ddp_sequential:
-        plugins = DDPSequentialPlugin()
+    if args.use_rpc_sequential:
+        plugins = RPCSequentialPlugin()
 
     model = LitResnet(batch_size=args.batch_size, manual_optimization=not args.automatic_optimization)
 
@@ -219,6 +222,6 @@ if __name__ == "__main__":
     trainer.fit(model, cifar10_dm)
     trainer.test(model, datamodule=cifar10_dm)
 
-    if trainer.accelerator_backend.rpc_enabled:
+    if trainer.accelerator.rpc_enabled:
         # Called at the end of trainer to ensure all processes are killed
-        trainer.accelerator_backend.ddp_plugin.exit_rpc_process()
+        trainer.training_type_plugin.exit_rpc_process()
