@@ -63,16 +63,16 @@ class ImageNetLightningModel(LightningModule):
     )
 
     def __init__(
-            self,
-            data_path: str,
-            arch: str = 'resnet18',
-            pretrained: bool = False,
-            lr: float = 0.1,
-            momentum: float = 0.9,
-            weight_decay: float = 1e-4,
-            batch_size: int = 4,
-            workers: int = 2,
-            **kwargs,
+        self,
+        data_path: str,
+        arch: str = 'resnet18',
+        pretrained: bool = False,
+        lr: float = 0.1,
+        momentum: float = 0.9,
+        weight_decay: float = 1e-4,
+        batch_size: int = 4,
+        workers: int = 2,
+        **kwargs,
     ):
         super().__init__()
         self.save_hyperparameters()
@@ -109,7 +109,7 @@ class ImageNetLightningModel(LightningModule):
         self.log('val_acc5', acc5, on_step=True, on_epoch=True)
 
     @staticmethod
-    def __accuracy(output, target, topk=(1,)):
+    def __accuracy(output, target, topk=(1, )):
         """Computes the accuracy over the k top predictions for the specified values of k"""
         with torch.no_grad():
             maxk = max(topk)
@@ -126,16 +126,8 @@ class ImageNetLightningModel(LightningModule):
             return res
 
     def configure_optimizers(self):
-        optimizer = optim.SGD(
-            self.parameters(),
-            lr=self.lr,
-            momentum=self.momentum,
-            weight_decay=self.weight_decay
-        )
-        scheduler = lr_scheduler.LambdaLR(
-            optimizer,
-            lambda epoch: 0.1 ** (epoch // 30)
-        )
+        optimizer = optim.SGD(self.parameters(), lr=self.lr, momentum=self.momentum, weight_decay=self.weight_decay)
+        scheduler = lr_scheduler.LambdaLR(optimizer, lambda epoch: 0.1**(epoch // 30))
         return [optimizer], [scheduler]
 
     def train_dataloader(self):
@@ -152,7 +144,8 @@ class ImageNetLightningModel(LightningModule):
                 transforms.RandomHorizontalFlip(),
                 transforms.ToTensor(),
                 normalize,
-            ]))
+            ])
+        )
 
         train_loader = torch.utils.data.DataLoader(
             dataset=train_dataset,
@@ -169,12 +162,15 @@ class ImageNetLightningModel(LightningModule):
         )
         val_dir = os.path.join(self.data_path, 'val')
         val_loader = torch.utils.data.DataLoader(
-            datasets.ImageFolder(val_dir, transforms.Compose([
-                transforms.Resize(256),
-                transforms.CenterCrop(224),
-                transforms.ToTensor(),
-                normalize,
-            ])),
+            datasets.ImageFolder(
+                val_dir,
+                transforms.Compose([
+                    transforms.Resize(256),
+                    transforms.CenterCrop(224),
+                    transforms.ToTensor(),
+                    normalize,
+                ])
+            ),
             batch_size=self.batch_size,
             shuffle=False,
             num_workers=self.workers,
@@ -203,26 +199,40 @@ class ImageNetLightningModel(LightningModule):
     @staticmethod
     def add_model_specific_args(parent_parser):  # pragma: no-cover
         parser = ArgumentParser(parents=[parent_parser])
-        parser.add_argument('-a', '--arch', metavar='ARCH', default='resnet18',
-                            choices=ImageNetLightningModel.MODEL_NAMES,
-                            help=('model architecture: ' + ' | '.join(ImageNetLightningModel.MODEL_NAMES)
-                                  + ' (default: resnet18)'))
-        parser.add_argument('-j', '--workers', default=4, type=int, metavar='N',
-                            help='number of data loading workers (default: 4)')
-        parser.add_argument('-b', '--batch-size', default=256, type=int,
-                            metavar='N',
-                            help='mini-batch size (default: 256), this is the total '
-                                 'batch size of all GPUs on the current node when '
-                                 'using Data Parallel or Distributed Data Parallel')
-        parser.add_argument('--lr', '--learning-rate', default=0.1, type=float,
-                            metavar='LR', help='initial learning rate', dest='lr')
-        parser.add_argument('--momentum', default=0.9, type=float, metavar='M',
-                            help='momentum')
-        parser.add_argument('--wd', '--weight-decay', default=1e-4, type=float,
-                            metavar='W', help='weight decay (default: 1e-4)',
-                            dest='weight_decay')
-        parser.add_argument('--pretrained', dest='pretrained', action='store_true',
-                            help='use pre-trained model')
+        parser.add_argument(
+            '-a',
+            '--arch',
+            metavar='ARCH',
+            default='resnet18',
+            choices=ImageNetLightningModel.MODEL_NAMES,
+            help=('model architecture: ' + ' | '.join(ImageNetLightningModel.MODEL_NAMES) + ' (default: resnet18)')
+        )
+        parser.add_argument(
+            '-j', '--workers', default=4, type=int, metavar='N', help='number of data loading workers (default: 4)'
+        )
+        parser.add_argument(
+            '-b',
+            '--batch-size',
+            default=256,
+            type=int,
+            metavar='N',
+            help='mini-batch size (default: 256), this is the total batch size of all GPUs on the current node'
+            ' when using Data Parallel or Distributed Data Parallel'
+        )
+        parser.add_argument(
+            '--lr', '--learning-rate', default=0.1, type=float, metavar='LR', help='initial learning rate', dest='lr'
+        )
+        parser.add_argument('--momentum', default=0.9, type=float, metavar='M', help='momentum')
+        parser.add_argument(
+            '--wd',
+            '--weight-decay',
+            default=1e-4,
+            type=float,
+            metavar='W',
+            help='weight decay (default: 1e-4)',
+            dest='weight_decay'
+        )
+        parser.add_argument('--pretrained', dest='pretrained', action='store_true', help='use pre-trained model')
         return parser
 
 
@@ -249,12 +259,11 @@ def main(args: Namespace) -> None:
 def run_cli():
     parent_parser = ArgumentParser(add_help=False)
     parent_parser = pl.Trainer.add_argparse_args(parent_parser)
-    parent_parser.add_argument('--data-path', metavar='DIR', type=str,
-                               help='path to dataset')
-    parent_parser.add_argument('-e', '--evaluate', dest='evaluate', action='store_true',
-                               help='evaluate model on validation set')
-    parent_parser.add_argument('--seed', type=int, default=42,
-                               help='seed for initializing training.')
+    parent_parser.add_argument('--data-path', metavar='DIR', type=str, help='path to dataset')
+    parent_parser.add_argument(
+        '-e', '--evaluate', dest='evaluate', action='store_true', help='evaluate model on validation set'
+    )
+    parent_parser.add_argument('--seed', type=int, default=42, help='seed for initializing training.')
     parser = ImageNetLightningModel.add_model_specific_args(parent_parser)
     parser.set_defaults(
         profiler="simple",
