@@ -14,7 +14,7 @@
 from pytorch_lightning.core.lightning import LightningModule
 from pytorch_lightning.utilities import rank_zero_warn
 from pytorch_lightning.utilities.exceptions import MisconfigurationException
-from pytorch_lightning.utilities.model_utils import is_overridden
+from pytorch_lightning.utilities.model_helpers import is_overridden
 
 
 class ConfigValidator(object):
@@ -78,8 +78,8 @@ class ConfigValidator(object):
         has_overriden_optimization_functions = trainer.overriden_optimizer_step or trainer.overriden_optimizer_zero_grad
         if (has_overriden_optimization_functions) and going_to_accumulate_grad_batches and automatic_optimization:
             raise MisconfigurationException(
-                'When overriding `LightningModule` optimizer_step or optimizer_zero_grad'
-                ' , `accumulate_grad_batches` in `Trainer` should to be 1.'
+                'When overriding `LightningModule` optimizer_step or optimizer_zero_grad,'
+                ' `accumulate_grad_batches` in `Trainer` should be 1.'
                 ' It ensures optimizer_step or optimizer_zero_grad are called on every batch.'
             )
 
@@ -95,10 +95,6 @@ class ConfigValidator(object):
         has_step = is_overridden(step_name, model)
 
         if has_loader and not has_step:
-            rank_zero_warn(
-                f'you passed in a {loader_name} but have no {step_name}. Skipping {eval_loop_name} loop'
-            )
+            rank_zero_warn(f'you passed in a {loader_name} but have no {step_name}. Skipping {eval_loop_name} loop')
         if has_step and not has_loader:
-            rank_zero_warn(
-                f'you defined a {step_name} but have no {loader_name}. Skipping {eval_loop_name} loop'
-            )
+            rank_zero_warn(f'you defined a {step_name} but have no {loader_name}. Skipping {eval_loop_name} loop')
