@@ -11,8 +11,8 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from pytorch_lightning.utilities.exceptions import MisconfigurationException
 from pytorch_lightning.callbacks import GradientAccumulationScheduler
+from pytorch_lightning.utilities.exceptions import MisconfigurationException
 
 
 class TrainingTricksConnector:
@@ -21,12 +21,12 @@ class TrainingTricksConnector:
         self.trainer = trainer
 
     def on_trainer_init(
-            self,
-            gradient_clip_val,
-            track_grad_norm,
-            accumulate_grad_batches,
-            truncated_bptt_steps,
-            terminate_on_nan
+        self,
+        gradient_clip_val,
+        track_grad_norm,
+        accumulate_grad_batches,
+        truncated_bptt_steps,
+        terminate_on_nan,
     ):
 
         self.trainer.terminate_on_nan = terminate_on_nan
