@@ -11,10 +11,8 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
 """Helper functions to help with reproducibility of models. """
 
-import logging
 import os
 import random
 from typing import Optional
@@ -22,10 +20,9 @@ from typing import Optional
 import numpy as np
 import torch
 
+from pytorch_lightning import _logger as log
 from pytorch_lightning.utilities import rank_zero_warn
 
-log = logging.getLogger(__name__)
-
 
 def seed_everything(seed: Optional[int] = None) -> int:
     """
